@@ -1404,6 +1404,16 @@ makeprint(struct str const *in, struct str *out)
                wc = isuni ? 0x2421 : '?';
             else
                wc = isuni ? 0x2426 : '?';
+         }else if(isuni){ /* TODO ctext */
+            /* We need to actively filter out L-TO-R and R-TO-R marks TODO ctext */
+            if(wc == 0x200E || wc == 0x200F || (wc >= 0x202A && wc <= 0x202E))
+               continue;
+            /* And some zero-width messes */
+            if(wc >= 0x200B && wc <= 0x200D)
+               continue;
+            /* Oh about the ISO C wide character interfaces, baby! */
+            if(wc == 0xFEFF)
+               continue;
          }
          if ((n = wctomb(mbb, wc)) <= 0)
             continue;
@@ -1839,8 +1849,8 @@ n_verr(char const *format, va_list ap)
 {
    /* Check use cases of PS_ERRORS_NOTED, too! */
 #ifdef HAVE_ERRORS
-   char buf[LINESIZE], *xbuf = buf;
-   int l = sizeof(buf) -1;
+   char buf[LINESIZE], *xbuf;
+   int lmax, l;
    struct err_node *enp;
 
    LCTA(ERRORS_MAX > 3);
@@ -1856,20 +1866,17 @@ n_verr(char const *format, va_list ap)
    }
 
 #ifdef HAVE_ERRORS
+   xbuf = buf;
+   lmax = sizeof buf;
 jredo:
-   l = vsnprintf(xbuf, l +1, format, ap);
-   if (l == 0) {
-      assert(xbuf == buf);
+   l = vsnprintf(xbuf, lmax, format, ap);
+   if (l <= 0)
       goto jleave;
+   if (UICMP(z, l, >=, lmax)) {
+      lmax = ++l;
+      xbuf = srealloc((xbuf == buf ? NULL : xbuf), lmax);
+      goto jredo;
    }
-   if (UICMP(z, l, >=, sizeof buf)) {
-      if (xbuf == buf) {
-         xbuf = smalloc(l +1);
-         goto jredo;
-      }
-      l = sizeof(buf) -1;
-   }
-   xbuf[l] = '\0';
 
    fwrite(xbuf, 1, l, stderr);
 
