@@ -145,6 +145,10 @@ option_update() {
    if feat_no SOCKETS; then
       WANT_MD5=0
    fi
+
+   if feat_yes DEVEL; then
+      WANT_DEBUG=1
+   fi
    if feat_yes DEBUG; then
       WANT_NOALLOCA=1 WANT_DEVEL=1
    fi
@@ -168,6 +172,8 @@ os_early_setup() {
 }
 
 os_setup() {
+   # OSFULLSPEC is used to recognize changes (i.e., machine type, updates etc.)
+   OSFULLSPEC="${OS:-`uname -a | ${tr} '[A-Z]' '[a-z]'`}"
    OS="${OS:-`uname -s | ${tr} '[A-Z]' '[a-z]'`}"
    msg 'Operating system is "%s"' ${OS}
 
@@ -267,6 +273,7 @@ cc_setup() {
       CFLAGS= LDFLAGS=
       export CFLAGS LDFLAGS
    fi
+
    [ -n "${CC}" ] && [ "${CC}" != cc ] && { _cc_default; return; }
 
    printf >&2 'Searching for a usable C compiler .. $CC='
@@ -278,6 +285,8 @@ cc_setup() {
       CC=${i}
    elif { i="`command -v tcc`"; }; then
       CC=${i}
+   elif { i="`command -v pcc`"; }; then
+      CC=${i}
    else
       if [ "${CC}" = cc ]; then
          :
@@ -291,7 +300,7 @@ cc_setup() {
          config_exit 1
       fi
    fi
-   printf >&2 '"%s"\n' "${CC}"
+   printf >&2 -- '"%s"\n' "${CC}"
    export CC
 }
 
@@ -318,10 +327,16 @@ cc_flags() {
          msg 'Testing usable $CFLAGS/$LDFLAGS for $CC="%s"' "${CC}"
       fi
 
-      if [ "${CC}" = tcc ]; then
+      i=`echo "${CC}" | ${awk} 'BEGIN{FS="/"}{print $NF}'`
+      if { echo "${i}" | ${grep} tcc; } >/dev/null 2>&1; then
          msg ' . have special tcc(1) environmental rules ...'
          _cc_flags_tcc
       else
+         # As of pcc CVS 2016-04-02, stack protection support is announced but
+         # will break if used on Linux
+         if { echo "${i}" | ${grep} pcc; } >/dev/null 2>&1; then
+            force_no_stackprot=1
+         fi
          _cc_flags_generic
       fi
 
@@ -346,7 +361,7 @@ _cc_flags_tcc() {
    cc_check -pedantic
 
    if feat_yes DEBUG; then
-      cc_check -b
+      # May have problems to find libtcc cc_check -b
       cc_check -g
    fi
 
@@ -359,19 +374,36 @@ _cc_flags_generic() {
    _CFLAGS= _LDFLAGS=
    feat_yes DEVEL && cc_check -std=c89 || cc_check -std=c99
 
-   cc_check -Wall
-   cc_check -Wextra
-   cc_check -pedantic
+   # Check -g first since some others may rely upon -g / optim. level
+   if feat_yes DEBUG; then
+      cc_check -O
+      cc_check -g
+   elif [ ${cc_maxopt} -gt 2 ] && cc_check -O3; then
+      :
+   elif [ ${cc_maxopt} -gt 1 ] && cc_check -O2; then
+      :
+   elif [ ${cc_maxopt} -gt 0 ] && cc_check -O1; then
+      :
+   else
+      cc_check -O
+   fi
 
-   cc_check -Wbad-function-cast
-   cc_check -Wcast-align
-   cc_check -Wcast-qual
-   cc_check -Winit-self
-   cc_check -Wmissing-prototypes
-   cc_check -Wshadow
-   cc_check -Wunused
-   cc_check -Wwrite-strings
-   cc_check -Wno-long-long
+   if feat_yes DEVEL && cc_check -Weverything; then
+      :
+   else
+      cc_check -Wall
+      cc_check -Wextra
+      cc_check -Wbad-function-cast
+      cc_check -Wcast-align
+      cc_check -Wcast-qual
+      cc_check -Winit-self
+      cc_check -Wmissing-prototypes
+      cc_check -Wshadow
+      cc_check -Wunused
+      cc_check -Wwrite-strings
+      cc_check -Wno-long-long
+   fi
+   cc_check -pedantic
 
    if feat_yes AMALGAMATION && feat_no DEVEL; then
       cc_check -Wno-unused-function
@@ -393,7 +425,7 @@ _cc_flags_generic() {
          fi
       else
          msg 'Not checking for -fstack-protector compiler option,'
-         msg 'since that caused linker errors in a "similar" configuration.'
+         msg 'since that caused errors in a "similar" configuration.'
          msg 'You may turn off WANT_AUTOCC and use your own settings, rerun'
       fi
    fi
@@ -401,17 +433,20 @@ _cc_flags_generic() {
    if feat_yes AMALGAMATION; then
       cc_check -pipe
    fi
-   if feat_yes DEBUG; then
-      cc_check -O
-      cc_check -g
-   elif [ ${cc_maxopt} -gt 2 ] && cc_check -O3; then
-      :
-   elif [ ${cc_maxopt} -gt 1 ] && cc_check -O2; then
-      :
-   elif [ ${cc_maxopt} -gt 0 ] && cc_check -O1; then
-      :
-   else
-      cc_check -O
+
+   # LD (+ dependend CC)
+
+   if feat_yes DEVEL; then
+      _ccfg=${_CFLAGS}
+      # -fsanitize=address
+      #if cc_check -fsanitize=memory &&
+      #      ld_check -fsanitize=memory &&
+      #      cc_check -fsanitize-memory-track-origins=2 &&
+      #      ld_check -fsanitize-memory-track-origins=2; then
+      #   :
+      #else
+      #   _CFLAGS=${_ccfg}
+      #fi
    fi
 
    ld_check -Wl,-z,relro
@@ -448,7 +483,7 @@ config_exit() {
 msg() {
    fmt=${1}
    shift
-   printf >&2 "${fmt}\\n" "${@}"
+   printf >&2 -- "${fmt}\\n" "${@}"
 }
 
 ## First of all, create new configuration and check wether it changed
@@ -566,8 +601,8 @@ trap "${rm} -f ${tmp}" EXIT
 printf >&2 'Reading and preparing configuration from "%s" ... ' ${rc}
 ${rm} -f ${tmp}
 # We want read(1) to perform backslash escaping in order to be able to use
-# multiline values in make.rc; GNU sed(1) blew me off the map, being VERY slow,
-# first shell / awk mixed case better, Aahron Robbins suggested the following
+# multiline values in make.rc; the resulting sh(1)/sed(1) code was very slow in
+# VMs (see [fa2e248]), Aharon Robbins suggested the following
 < ${rc} ${awk} 'BEGIN{line = ""}{
    gsub(/^[[:space:]]+/, "", $0)
    gsub(/[[:space:]]+$/, "", $0)
@@ -784,13 +819,21 @@ cc_setup
 
 ${cat} > ${tmp}.c << \!
 #include <stdio.h>
-int main(int argc, char **argv)
-{
+#include <string.h>
+static void doit(char const *s);
+int
+main(int argc, char **argv){
    (void)argc;
    (void)argv;
-   puts("Hello world");
+   doit("Hello world");
    return 0;
 }
+static void
+doit(char const *s){
+   char buf[12];
+   strcpy(buf, s);
+   puts(s);
+}
 !
 
 if "${CC}" ${INCS} ${CFLAGS} ${ADDCFLAGS} ${LDFLAGS} ${ADDLDFLAGS} \
@@ -805,7 +848,7 @@ else
 fi
 
 cc_check() {
-   [ -n "${cc_check_silent}" ] || printf >&2 ' . %s .. ' "${1}"
+   [ -n "${cc_check_silent}" ] || printf >&2 ' . CC %s .. ' "${1}"
    if "${CC}" ${INCS} ${_CFLAGS} ${1} ${ADDCFLAGS} ${_LDFLAGS} ${ADDLDFLAGS} \
          -o ${tmp2} ${tmp}.c ${LIBS} >/dev/null 2>&1; then
       _CFLAGS="${_CFLAGS} ${1}"
@@ -817,7 +860,7 @@ cc_check() {
 }
 
 ld_check() {
-   [ -n "${cc_check_silent}" ] || printf >&2 ' . %s .. ' "${1}"
+   [ -n "${cc_check_silent}" ] || printf >&2 ' . LD %s .. ' "${1}"
    if "${CC}" ${INCS} ${_CFLAGS} ${_LDFLAGS} ${1} ${ADDLDFLAGS} \
          -o ${tmp2} ${tmp}.c ${LIBS} >/dev/null 2>&1; then
       _LDFLAGS="${_LDFLAGS} ${1}"
@@ -834,17 +877,18 @@ for i in \
       INCS LIBS \
       ; do
    eval j=\$${i}
-   printf "${i}=${j}\n" >> ${newlst}
+   printf -- "${i}=${j}\n" >> ${newlst}
 done
 for i in \
       CC \
       CFLAGS \
       LDFLAGS \
       PATH C_INCLUDE_PATH LD_LIBRARY_PATH \
+      OSFULLSPEC \
       ; do
    eval j=\$${i}
-   printf "${i} = ${j}\n" >> ${newmk}
-   printf "${i}=${j}\n" >> ${newlst}
+   printf -- "${i} = ${j}\n" >> ${newmk}
+   printf -- "${i}=${j}\n" >> ${newlst}
 done
 
 # Now finally check wether we already have a configuration and if so, wether
@@ -889,13 +933,13 @@ msg() {
    fmt=${1}
    shift
    printf "*** ${fmt}\\n" "${@}"
-   printf "${fmt}\\n" "${@}" >&5
+   printf -- "${fmt}\\n" "${@}" >&5
 }
 msg_nonl() {
    fmt=${1}
    shift
    printf "*** ${fmt}\\n" "${@}"
-   printf "${fmt}" "${@}" >&5
+   printf -- "${fmt}" "${@}" >&5
 }
 
 exec 5>&2 > ${log} 2>&1
@@ -984,7 +1028,7 @@ run_check() {
 ##
 
 # May be multiline..
-[ -n "${OS_DEFINES}" ] && printf "${OS_DEFINES}" >> ${h}
+[ -n "${OS_DEFINES}" ] && printf -- "${OS_DEFINES}" >> ${h}
 
 if run_check clock_gettime 'clock_gettime(2)' \
    '#define HAVE_CLOCK_GETTIME' << \!
@@ -1087,6 +1131,21 @@ else
    config_exit 1
 fi
 
+if link_check environ 'environ(3)' << \!
+#include <stdio.h> /* For C89 NULL */
+int main(void){
+   extern char **environ;
+
+   return environ[0] == NULL;
+}
+!
+then
+   :
+else
+   msg 'ERROR: we require the environ(3) array for subprocess control.'
+   config_exit 1
+fi
+
 if link_check termios 'termios.h and tc*(3) family' << \!
 #include <termios.h>
 int main(void){
