@@ -241,10 +241,16 @@ _pipefile(char const *pipecomm, struct mimepart const *mpp, FILE **qbuf,
 #ifdef HAVE_FILTER_HTML_TAGSOUP
    if (pipecomm == MIME_TYPE_HANDLER_HTML) {
       union {int (*ptf)(void); char const *sh;} u;
-      u.ptf = &htmlflt_process_main;
-      rbuf = Popen((char*)-1, "W", u.sh, NULL, fileno(*qbuf));
+
+      fflush(*qbuf);
+      if (*qbuf != stdout) /* xxx never?  v15: it'll be a filter anyway */
+         fflush(stdout);
+
       pipecomm = "Builtin HTML tagsoup filter";
-      goto jafter_tagsoup_hack;
+      u.ptf = &htmlflt_process_main;
+      if((rbuf = Popen((char*)-1, "W", u.sh, NULL, fileno(*qbuf))) == NULL)
+         goto jerror;
+      goto jleave;
    }
 #endif
 
@@ -279,17 +285,20 @@ _pipefile(char const *pipecomm, struct mimepart const *mpp, FILE **qbuf,
       sh = XSHELL;
 
    rbuf = Popen(pipecomm, "W", sh, env_addon, (async ? -1 : fileno(*qbuf)));
+   if (rbuf == NULL) {
 #ifdef HAVE_FILTER_HTML_TAGSOUP
-jafter_tagsoup_hack:
+jerror:
 #endif
-   if (rbuf == NULL)
       n_err(_("Cannot run MIME type handler \"%s\": %s\n"),
          pipecomm, strerror(errno));
-   else {
+   } else {
       fflush(*qbuf);
       if (*qbuf != stdout)
          fflush(stdout);
    }
+#ifdef HAVE_FILTER_HTML_TAGSOUP
+jleave:
+#endif
    NYD_LEAVE;
    return rbuf;
 }
@@ -1076,12 +1085,18 @@ newfile(struct mimepart *ip, int *ispipe)
    }
 
    if (options & OPT_INTERACTIVE) {
+      struct str prompt;
       char *f2, *f3;
+
+      /* TODO Generic function which asks for filename.
+       * TODO If the current part is the first textpart the target
+       * TODO is implicit from outer `write' etc! */
+      /* I18N: Filename input prompt with file type indication */
+      str_concat_csvl(&prompt, _("Enter filename for part "),
+         (ip->m_partstring != NULL) ? ip->m_partstring : _("?"),
+         _(" ("), ip->m_ct_type_plain, _("): "), NULL);
 jgetname:
-      printf(_("Enter filename for part %s (%s)"),
-         (ip->m_partstring != NULL) ? ip->m_partstring : "?",
-         ip->m_ct_type_plain);
-      f2 = n_input_cp_addhist(": ", ((f != (char*)-1 && f != NULL)
+      f2 = n_input_cp_addhist(prompt.s, ((f != (char*)-1 && f != NULL)
             ? fexpand_nshell_quote(f) : NULL), TRU1);
       if (f2 == NULL || *f2 == '\0') {
          if (options & OPT_D_V)
