@@ -880,7 +880,7 @@ _ncl_term_mode(bool_t raw)
    tiosp->c_iflag &= ~(ISTRIP);
    tiosp->c_lflag &= ~(ECHO /*| ECHOE | ECHONL */| ICANON | IEXTEN);
 jleave:
-   tcsetattr(STDIN_FILENO, TCSAFLUSH, tiosp);
+   tcsetattr(STDIN_FILENO, TCSADRAIN, tiosp);
    NYD2_LEAVE;
 }
 
@@ -891,7 +891,7 @@ _ncl_check_grow(struct line *l, size_t no SMALLOC_DEBUG_ARGS)
    NYD2_ENTER;
 
    i = (l->topins + no) * sizeof(struct cell) + 2 * sizeof(struct cell);
-   if (i > *l->x_bufsize) {
+   if (i >= *l->x_bufsize) {
       i <<= 1;
       *l->x_bufsize = i;
       l->line.cbuf =
@@ -1843,11 +1843,13 @@ tty_destroy(void)
    if (v == NULL)
       goto jleave;
 
-   if ((hp = _ncl_hist) != NULL)
-      while (hp->older != NULL && hs-- != 0)
-         hp = hp->older;
    dogabby = ok_blook(history_gabby_persist);
 
+   if ((hp = _ncl_hist) != NULL)
+      for (; hp->older != NULL; hp = hp->older)
+         if ((dogabby || !hp->isgabby) && --hs == 0)
+            break;
+
    hold_all_sigs(); /* TODO too heavy, yet we may jump even here!? */
    f = fopen(v, "w"); /* TODO temporary + rename?! */
    if (f == NULL)
